#include "stack.hpp"
#include <brick-unit>

int main()
{
    using namespace bast;

    brq::test_case( "push_1" ) = []
    {
        basic_stack s;
        s.push( 7 );
        ASSERT_EQ( s.pop< int >(), 7 );
    };

    brq::test_case( "push_8" ) = []
    {
        basic_stack s;

        for ( int i = 0; i <= 7; ++ i )
            s.push( 7 + i );

        for ( int i = 7; i >= 0; -- i )
            ASSERT_EQ( s.pop< int >(), 7 + i );
    };

    brq::test_case( "push_32" ) = []
    {
        basic_stack s;

        for ( int i = 0; i <= 31; ++ i )
        {
            s.push( 7 + i );
            s.push( 7 + i );
            s.pop< int >();
        }

        for ( int i = 31; i >= 0; -- i )
            ASSERT_EQ( s.pop< int >(), 7 + i );
    };

    brq::test_case( "push_var" ) = []
    {
        basic_stack s;
        s.push( uint16_t( 3 ) );
        s.push( uint8_t( 12 ) );
        ASSERT_EQ( s.pop< uint8_t >(), 12 );
        ASSERT_EQ( s.pop< uint16_t >(), 3 );
    };

    brq::test_case( "align" ) = []
    {
        basic_stack s;

        for ( int i = 0; i < 6; ++i )
        {
            s.push( i );
            ASSERT_EQ( uintptr_t( s.top_ptr() ) % 4, 0 );
        }

        s.push< uint8_t >( 33 );
        s.push< uint32_t >( 33 );
        ASSERT_EQ( uintptr_t( s.top_ptr() ) % 4, 0 );
        s.pop< uint32_t >();
        ASSERT_EQ( uintptr_t( s.top_ptr() ) % 4, 0 );
    };

    brq::test_case( "print" ) = []
    {
        basic_stack s;
        ASSERT_EQ( brq::format( s ).data(), "|⟩" );

        for ( int i = 0; i < 5; ++i )
        {
            s.push( uint16_t( 3 + i ) );
            s.push( uint8_t( 12 + i ) );
        }

        ASSERT_EQ( brq::format( s ).data(), "| 3 12 4 13 5 14 6 15 7 16 ⟩" );
    };
}
